package endpoint

import (
	"fmt"

	pb "github.com/bestnite/go-igdb/proto"
	"github.com/go-resty/resty/v2"

	"google.golang.org/protobuf/proto"
)

type EventLogos struct {
	BaseEndpoint[pb.EventLogo]
}

func NewEventLogos(request func(URL string, dataBody any) (*resty.Response, error)) *EventLogos {
	a := &EventLogos{
		BaseEndpoint[pb.EventLogo]{
			endpointName: EPEventLogos,
			request:      request,
		},
	}
	a.queryFunc = a.Query
	return a
}

func (a *EventLogos) Query(query string) ([]*pb.EventLogo, error) {
	resp, err := a.request("https://api.igdb.com/v4/event_logos.pb", query)
	if err != nil {
		return nil, fmt.Errorf("failed to request: %w", err)
	}

	data := pb.EventLogoResult{}
	if err = proto.Unmarshal(resp.Body(), &data); err != nil {
		return nil, fmt.Errorf("failed to unmarshal: %w", err)
	}

	if len(data.Eventlogos) == 0 {
		return nil, fmt.Errorf("no results: %s", query)
	}

	return data.Eventlogos, nil
}
